@@ -43,24 +43,6 @@
 
 G_DEFINE_BOXED_TYPE (GITypelib, gi_typelib, gi_typelib_ref, gi_typelib_unref)
 
-GIInfoType
-gi_typelib_blob_type_to_info_type (GITypelibBlobType blob_type)
-{
-  switch (blob_type)
-    {
-    case BLOB_TYPE_BOXED:
-      /* `BLOB_TYPE_BOXED` now always refers to a `StructBlob`, and
-       * `GIRegisteredTypeInfo` (the parent type of `GIStructInfo`) has a method
-       * for distinguishing whether the struct is a boxed type. So presenting
-       * `BLOB_TYPE_BOXED` as its own `GIBaseInfo` subclass is not helpful.
-       * See commit e28078c70cbf4a57c7dbd39626f43f9bd2674145 and
-       * https://gitlab.gnome.org/GNOME/glib/-/issues/3245. */
-      return GI_INFO_TYPE_STRUCT;
-    default:
-      return (GIInfoType) blob_type;
-    }
-}
-
 typedef struct {
   GITypelib *typelib;
   GSList *context_stack;
@@ -105,7 +87,7 @@ get_dir_entry_checked (GITypelib *typelib,
       return FALSE;
     }
 
-  offset = header->directory + (index - 1) * header->entry_blob_size;
+  offset = header->directory + (index - 1u) * header->entry_blob_size;
 
   if (typelib->len < offset + sizeof (DirEntry))
     {
@@ -179,7 +161,8 @@ gi_typelib_get_dir_entry (GITypelib *typelib,
 {
   Header *header = (Header *)typelib->data;
 
-  return (DirEntry *)&typelib->data[header->directory + (index - 1) * header->entry_blob_size];
+  /* this deliberately doesn’t check for underflow of @index; see get_dir_entry_checked() for that */
+  return (DirEntry *)&typelib->data[header->directory + (index - 1u) * header->entry_blob_size];
 }
 
 static Section *
@@ -321,7 +304,7 @@ strsplit_iter_next (StrSplitIter  *iter,
   if (next)
     {
       iter->s = next + iter->sep_len;
-      len = next - s;
+      len = (size_t) (next - s);
     }
   else
     {
@@ -334,7 +317,8 @@ strsplit_iter_next (StrSplitIter  *iter,
     }
   else
     {
-      g_string_overwrite_len (&iter->buf, 0, s, (gssize)len);
+      g_string_overwrite_len (&iter->buf, 0, s, (gssize)len + 1);
+      iter->buf.str[len] = '\0';
       *out_val = iter->buf.str;
     }
   return TRUE;
@@ -1735,7 +1719,7 @@ validate_object_blob (ValidateContext *ctx,
     }
 
   if (typelib->len < offset + sizeof (ObjectBlob) +
-            (blob->n_interfaces + blob->n_interfaces % 2) * 2 +
+            (blob->n_interfaces + blob->n_interfaces % 2u) * 2u +
             blob->n_fields * sizeof (FieldBlob) +
             blob->n_properties * sizeof (PropertyBlob) +
             blob->n_methods * sizeof (FunctionBlob) +
@@ -1892,7 +1876,7 @@ validate_interface_blob (ValidateContext *ctx,
     return FALSE;
 
   if (typelib->len < offset + sizeof (InterfaceBlob) +
-            (blob->n_prerequisites + blob->n_prerequisites % 2) * 2 +
+            (blob->n_prerequisites + blob->n_prerequisites % 2u) * 2u +
             blob->n_properties * sizeof (PropertyBlob) +
             blob->n_methods * sizeof (FunctionBlob) +
             blob->n_signals * sizeof (SignalBlob) +
